function HowTo_IterativelyEnhance
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% File name: HowTo_IterativelyEnhance.m
% proof-of-concept experiment for iterative phase-aware single-channel speech enhancement [1]
% Last update: 19.07.2017 
% Written by Pejman Mowlaee
% Email: pejman.mowlaee@tugraz.at
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% [1] P. Mowlaee and R. Saeidi, "Iterative Closed-Loop Phase-Aware Single-Channel Speech Enhancement," in IEEE Signal Processing Letters, vol. 20, no. 12, pp. 1235-1239, Dec. 2013.
% [2] R. C.  Hendriks, R. Heusdens and J. Jensen, "MMSE BASED NOISE PSD
%     TRACKING WITH LOW COMPLEXITY",IEEE International Conference on Acoustics, Speech and Signal Processing, 03/2010, Dallas, TX, p.4266-4269, (2010).
% [3] Y. Ephraim and D. Malah, “Speech enhancement using a minimum mean-square error log-spectral amplitude estimator,” IEEE Transactions on Acoustics, Speech, and Signal
%     Processing, vol. 33, pp. 443–445, Apr 1985.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Pejman Mowlaee, pejman.mowlaee@tugraz.at
% PhaseLab
% Signal Processing and Speech Communication Laboratory,
% Graz University of Technology, Austria
% (c) 2015
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc;close all;clear all;

param.fs=8e3; % sampling frequency
% read in audio files
[cleanwav0,fs1] =audioread(fullfile('Waves','Clean.wav')); cleanwav=resample(cleanwav0,param.fs,fs1);
[noisysignal0,fs2] =audioread(fullfile('Waves','Noisy.wav')); noisysignal=resample(noisysignal0,param.fs,fs2);

%


% algorithm parameters
param.snr_theshold_low_GDF=-15; % constraint for phase estimation
param.snr_theshold_high_GDF=6; % constraint for phase estimation
param.min_const=1e-4; % 1e-4
param.max_no_iter=6; % maximum number of iterations
param.NIF=20; % Number of first noise-only frames
param.nsmethod = 'Hendriks2010'; % 'Hendriks2010'/imcra ... noise estimation method [2]
param.ampmethod='MMSE-LSA';%  'PS', ML-LSA, ML-STSA (default); ... amplitude estimation method [3]
param.do_phase_est_blind = 1; % if 0: noise oracle
if ~param.do_phase_est_blind
    param.CleanSpeech = cleanwav; % only if param.do_phase_est_blind = 0
end

[enhsignal,param]=enh_pro_inout(noisysignal,param); % enhance as proposed in [1]

%%%%%%%%%%%%%%%%%%%%%%%%%%%% plotting %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
framelength = 40e-3*param.fs;
nfft = 1024;
lowerbound = -70;
upperbound = -30;


figure; 
subplot(221);  spectrogram(cleanwav/max(abs(cleanwav)),framelength,framelength*7/8,nfft,param.fs,'yaxis'); title('(a)'); 
caxis([lowerbound upperbound])
set(gca,'YTick',[0 1000 2000 3000 4000],'YTickLabel',[0 1 2 3 4])
ylabel('Frequency (kHz)')
box on
    
subplot(222); spectrogram(noisysignal0/max(abs(noisysignal)),framelength,framelength*7/8,nfft,param.fs,'yaxis'); title('(b)');  
set(gca,'YTick',[0 1000 2000 3000 4000],'YTickLabel',[0 1 2 3 4])
ylabel('Frequency (kHz)')
box on
    
subplot(223); spectrogram(enhsignal.sig_phaseaware/max(abs(enhsignal.sig_phaseaware)),framelength,framelength*7/8,nfft,param.fs,'yaxis');title('(c)');  
set(gca,'YTick',[0 1000 2000 3000 4000],'YTickLabel',[0 1 2 3 4])
caxis([lowerbound upperbound]);% colorbar;
ylabel('Frequency (kHz)')
box on
colormap(1-gray)    
    
subplot(2,2,4); plot(1:length(param.dist_e),param.dist_e/max(abs(param.dist_e)),'k'); title('(d)');  
set(gca,'TickLabelInterpreter','latex')
xlabel('Iterations')
ylabel('$D^{(i)}/D^{(1)}$','rotation',90)
xlim([1 length(param.dist_e)])
box on

end
